// core_base.shader

#if !defined(RUNTIME)
#   define VERTEX_SHADER
#endif

#if defined(VERTEX_SHADER)

uniform mat4 worldMat;
//uniform mat3 normalMat;
uniform mat4 viewMat;
//uniform mat4 cameraMat;
uniform mat4 projMat;

uniform lowp vec4 meshColor;

#if defined(SKIN)
attribute vec4 boneIndices;
attribute vec4 boneWeights;
uniform mat4 bones[46];
#endif

vec4 getPosWSLS(vec4 posLS)
{
#if defined(SKIN)
	vec4 p = vec4(0.0, 0.0, 0.0, 0.0);

	p += bones[int(boneIndices.x)] * posLS * boneWeights.x;
	p += bones[int(boneIndices.y)] * posLS * boneWeights.y;
	p += bones[int(boneIndices.z)] * posLS * boneWeights.z;
	p += bones[int(boneIndices.w)] * posLS * boneWeights.w;

	return p;
#else
	return worldMat * posLS;
#endif
}

vec4 getPosVSLS(vec4 posLS)
{
    return viewMat * (worldMat * posLS);
}

vec4 getPosVSWS(vec4 posWS)
{
    return viewMat * posWS;
}

vec4 getPosSSLS(vec4 posLS)
{
    return projMat * (viewMat * (worldMat * posLS));
}

vec4 getPosSSWS(vec4 posWS)
{
    return projMat * (viewMat * posWS);
}

vec4 getPosSSVS(vec4 posVS)
{
    return projMat * posVS;
}

vec3 getNrmWSLS(vec3 nrmLS)
{
#if defined(SKIN)
	vec4 p = vec4(0.0, 0.0, 0.0, 0.0);
	vec4 normal = vec4(nrmLS, 0.0);
	p += bones[int(boneIndices.x)] * normal * boneWeights.x;
	p += bones[int(boneIndices.y)] * normal * boneWeights.y;
	p += bones[int(boneIndices.z)] * normal * boneWeights.z;
	p += bones[int(boneIndices.w)] * normal * boneWeights.w;

	return normalize(p.xyz);

#endif
    return normalize( mat3(worldMat[0].xyz, worldMat[1].xyz, worldMat[2].xyz) * nrmLS);
//    return normalMat * vertNormal;
}

vec4 getVertexColor(vec3 vertexColor)
{
    return vec4(vertexColor.rgb/255.0, 1.0);
}

vec4 getVertexColor(vec4 vertexColor)
{
    return vertexColor/255.0;
}

mediump vec2 getEnvironmentTexCoord(vec3 nrmWS)
{
    vec3 nrmVS = normalize( mat3(viewMat[0].xyz, viewMat[1].xyz, viewMat[2].xyz) * nrmWS);
    nrmVS.y = -nrmVS.y;
    nrmVS.z = max(0.0, nrmVS.z);

    return normalize(nrmVS).xy * 0.5 + vec2(0.5);
}

#endif

lowp vec4 getSaturateColor(lowp vec4 color, lowp float saturateColor)
{
    lowp float greyScale = dot(color.rgb, vec3(0.2989, 0.5870, 0.1140));
    
    return vec4(mix(vec3(greyScale), color.rgb, saturateColor), color.a);
}
