// core_light.shader

#if defined(VERTEX_SHADER)

#define DIRECTIONAL_LIGHT
//#define POINT_LIGHT


#if defined(POINT_LIGHT) 

uniform highp vec4 light0Pos;
//uniform highp vec4 light1Pos;
//uniform highp vec4 light2Pos;
//uniform highp vec4 light3Pos;

//uniform highp vec4 light4Pos;
//uniform highp vec4 light5Pos;
//uniform highp vec4 light6Pos;
//uniform highp vec4 light7Pos;

lowp float getLightPoint1(vec3 posWS, vec3 nrmWS, highp vec4 light0Position)
{
    vec3 light0Dir = light0Position.xyz - posWS.xyz;

    lowp float len = length(light0Dir.xyz);

    lowp float v = max(dot(nrmWS, light0Dir.xyz)/len, 0.0);

    return (1.0 - min(len * light0Position.w, 1.0)) * v;
}

lowp vec2 getLightPoint2(vec3 posWS, vec3 nrmWS, highp vec4 light0Position, highp vec4 light1Position)
{
    vec3 light0Dir = light0Position.xyz - posWS.xyz;
    vec3 light1Dir = light1Position.xyz - posWS.xyz;

    lowp vec2 len = vec2( length(light0Dir.xyz),
        length(light1Dir.xyz) );

    lowp vec2 v = max( vec2(dot(nrmWS, light0Dir.xyz),
        dot(nrmWS, light1Dir.xyz)) / len, vec2(0.0) );

    return (vec2(1.0) - min(vec2(len * vec2(light0Position.w, light1Position.w)), vec2(1.0))) * v;
}

lowp vec4 getLightPoint3(vec3 posWS, vec3 nrmWS, highp vec4 light0Position, highp vec4 light1Position, highp vec4 light2Position)
{
    vec3 light0Dir = light0Position.xyz - posWS.xyz;
    vec3 light1Dir = light1Position.xyz - posWS.xyz;
    vec3 light2Dir = light2Position.xyz - posWS.xyz;

    lowp vec4 len = vec4( length(light0Dir.xyz),
        length(light1Dir.xyz),
        length(light2Dir.xyz),
        1.0 );

    lowp vec4 v = max( vec4(dot(nrmWS, light0Dir.xyz),
        dot(nrmWS, light1Dir.xyz),
        dot(nrmWS, light2Dir.xyz),
        0.0) / len, vec4(0.0) );

    return (vec4(1.0) - min(vec4(len * vec4(light0Position.w, light1Position.w, light2Position.w, 0.0)), vec4(1.0))) * v;
}

lowp vec4 getLightPoint4(vec3 posWS, vec3 nrmWS, highp vec4 light0Position, highp vec4 light1Position, highp vec4 light2Position, highp vec4 light3Position)
{
    vec3 light0Dir = light0Position.xyz - posWS.xyz;
    vec3 light1Dir = light1Position.xyz - posWS.xyz;
    vec3 light2Dir = light2Position.xyz - posWS.xyz;
    vec3 light3Dir = light3Position.xyz - posWS.xyz;

    lowp vec4 len = vec4( length(light0Dir.xyz),
        length(light1Dir.xyz),
        length(light2Dir.xyz),
        length(light3Dir.xyz) );

    lowp vec4 v = max( vec4(dot(nrmWS, light0Dir.xyz),
        dot(nrmWS, light1Dir.xyz),
        dot(nrmWS, light2Dir.xyz),
        dot(nrmWS, light3Dir.xyz)) / len, vec4(0.0) );

    return (vec4(1.0) - min(vec4(len * vec4(light0Position.w, light1Position.w, light2Position.w, light3Position.w)), vec4(1.0))) * v;
}

#elif defined(DIRECTIONAL_LIGHT) 

uniform highp vec3 light0Direction;
//uniform highp vec3 light1Direction;
//uniform highp vec3 light2Direction;
//uniform highp vec3 light3Direction;

//uniform highp vec3 light4Direction;
//uniform highp vec3 light5Direction;
//uniform highp vec3 light6Direction;
//uniform highp vec3 light7Direction;

lowp float getLightDirectional1(vec3 nrmWS, highp vec3 light0Dir)
{
    return max(dot(nrmWS, light0Dir), 0.0);
}

lowp vec2 getLightDirectional2(vec3 nrmWS, highp vec3 light0Dir, highp vec3 light1Dir)
{
    return max( vec2( dot(nrmWS, light0Dir),
        dot(nrmWS, light1Dir)), vec2(0.0) );
}

lowp vec4 getLightDirectional3(vec3 nrmWS, highp vec3 light0Dir, highp vec3 light1Dir, highp vec3 light2Dir)
{
    return max( vec4( dot(nrmWS, light0Dir),
        dot(nrmWS, light1Dir),
        dot(nrmWS, light2Dir),
        0.0), vec4(0.0) );
}

lowp vec4 getLightDirectional4(vec3 nrmWS, highp vec3 light0Dir, highp vec3 light1Dir, highp vec3 light2Dir, highp vec3 light3Dir)
{
    return max( vec4( dot(nrmWS, light0Dir),
        dot(nrmWS, light1Dir),
        dot(nrmWS, light2Dir),
        dot(nrmWS, light3Dir)), vec4(0.0) );
}

#endif         


uniform mediump vec3 light0Color;
//uniform mediump vec3 light1Color;
//uniform mediump vec3 light2Color;
//uniform mediump vec3 light3Color;

//uniform mediump vec3 light4Color;
//uniform mediump vec3 light5Color;
//uniform mediump vec3 light6Color;
//uniform mediump vec3 light7Color;

uniform lowp float materialEmit;

lowp vec3 getLight(vec3 posWS, vec3 nrmWS)
{
    lowp float intensity1;
//    lowp vec4 intensity4;
//    lowp vec2 intensity6;
//    lowp vec4 intensity7;
//    lowp vec4 intensity8;

#if defined(POINT_LIGHT) 
    intensity1 = getLightPoint1(posWS, nrmWS, light0Pos);
//    intensity4 = getLightPoint4(posWS, nrmWS, light0Pos, light1Pos, light2Pos, light3Pos);
//    intensity6 = getLightPoint2(posWS, nrmWS, light4Pos, light5Pos);
//    intensity7 = getLightPoint3(posWS, nrmWS, light4Pos, light5Pos, light6Pos);
//    intensity8 = getLightPoint4(posWS, nrmWS, light4Pos, light5Pos, light6Pos, light7Pos);
#elif defined(DIRECTIONAL_LIGHT)
    intensity1 = getLightDirectional1(nrmWS, light0Direction);
//    intensity4 = getLightDirectional4(nrmWS, light0Direction, light1Direction, light2Direction, light3Direction);
//    intensity6 = getLightDirectional2(nrmWS, light4Direction, light5Direction);
//    intensity7 = getLightDirectional3(nrmWS, light4Direction, light5Direction, light6Direction);
//    intensity8 = getLightDirectional4(nrmWS, light4Direction, light5Direction, light6Direction, light7Direction);
#else
    intensity1 = 1.0;
//    intensity4 = vec4(1.0);
//    intensity6 = vec2(1.0);
//    intensity7 = vec4(1.0);
//    intensity8 = vec4(1.0);
#endif

    vec3 lights = light0Color*intensity1;
//    lights += light1Color*intensity4.y + light2Color*intensity4.z + light3Color*intensity4.w;
//    lights += light4Color*intensity6.x + light5Color*intensity6.y;
//    lights += light4Color*intensity7.x + light5Color*intensity7.y + light6Color*intensity7.z;
//    lights += light4Color*intensity8.x + light5Color*intensity8.y + light6Color*intensity8.z + light7Color*intensity8.w;

    return mix(lights, vec3(1.0), materialEmit);
}

#endif
