// model.shader

#include "shaders/core_base.shader"
#include "shaders/core_fog.shader"
#include "shaders/core_light.shader"


varying mediump vec2 diffuseTexCoord;
varying mediump vec2 fogTexCoord;

varying lowp vec4 lightColor;


#if defined(VERTEX_SHADER)

attribute vec4 vertPos;
attribute vec3 vertColor;
attribute vec3 vertNormal;
attribute vec2 texCoords0;

void main()
{
    vec4 posWS = getPosWSLS(vertPos);
    vec4 posVS = getPosVSWS(posWS);
    gl_Position = getPosSSVS(posVS);

    vec3 nrmWS = getNrmWSLS(vertNormal);

    lowp vec4 vertexColor = getVertexColor(vertColor);
    lightColor = vec4(getLight(posWS.xyz, nrmWS) * vertexColor.rgb, vertexColor.a) * meshColor;

    diffuseTexCoord = texCoords0;
    fogTexCoord = getFogTexCoord(posVS);
}

#endif


#if defined(FRAGMENT_SHADER)

uniform sampler2D tex0; // Diffuse
uniform sampler2D tex1; // Fog

void main()
{
    lowp vec4 diffuseColor = texture2D(tex0, diffuseTexCoord);
    lowp vec4 fogColor = texture2D(tex1, fogTexCoord);

    lowp vec4 color = diffuseColor*lightColor;

    gl_FragColor = vec4(mix(color.rgb, fogColor.rgb, fogColor.a), color.a);
}

#endif
