// model_untex_unlit_ref_fog.shader

#include "shaders/core_base.shader"
#include "shaders/core_fog.shader"


varying mediump vec2 fogTexCoord;
varying mediump vec2 environmentTexCoord;

varying lowp vec4 diffuseColor;


#if defined(VERTEX_SHADER)

attribute vec4 vertPos;
attribute vec3 vertNormal;
attribute vec3 vertColor;

void main()
{
    vec4 posWS = getPosWSLS(vertPos);
    vec4 posVS = getPosVSWS(posWS);
    gl_Position = getPosSSVS(posVS);

    vec3 nrmWS = getNrmWSLS(vertNormal);
    
    diffuseColor = getVertexColor(vertColor) * meshColor;

    fogTexCoord = getFogTexCoord(posVS);
    environmentTexCoord = getEnvironmentTexCoord(nrmWS);
}

#endif


#if defined(FRAGMENT_SHADER)

uniform sampler2D tex0; // Environment
uniform sampler2D tex1; // Fog

void main()
{
    lowp vec4 environmentColor = texture2D(tex0, environmentTexCoord);
    lowp vec4 fogColor = texture2D(tex1, fogTexCoord);

    lowp vec4 color = diffuseColor;
    color.rgb += environmentColor.rgb;
//    color.rgb *= environmentColor.rgb + environmentColor.rgb;

    gl_FragColor = vec4(mix(color.rgb, fogColor.rgb, fogColor.a), color.a);
}

#endif
